@@ -4,7 +4,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-void zig_panic();
+void zig_panic(void);
 
 static void assert_or_panic(bool ok) {
     if (!ok) {
@@ -16,8 +16,17 @@ static void assert_or_panic(bool ok) {
 #  define ZIG_PPC32
 #endif
 
-#if defined __riscv && defined _ILP32
-#  define ZIG_RISCV32
+#ifdef __riscv
+#  ifdef _ILP32
+#    define ZIG_RISCV32
+#  else
+#    define ZIG_RISCV64
+#  endif
+#endif
+
+#if defined(__aarch64__) && defined(__linux__)
+// TODO: https://github.com/ziglang/zig/issues/14908
+#define ZIG_BUG_14908
 #endif
 
 #ifdef __i386__
@@ -60,6 +69,54 @@ static void assert_or_panic(bool ok) {
 #  define ZIG_NO_COMPLEX
 #endif
 
+#ifdef __x86_64__
+#define ZIG_NO_RAW_F16
+#endif
+
+#ifdef __i386__
+#define ZIG_NO_RAW_F16
+#endif
+
+#ifdef __mips__
+#define ZIG_NO_RAW_F16
+#endif
+
+#ifdef __riscv
+#define ZIG_NO_RAW_F16
+#endif
+
+#ifdef __wasm__
+#define ZIG_NO_RAW_F16
+#endif
+
+#ifdef __powerpc__
+#define ZIG_NO_RAW_F16
+#endif
+
+#ifdef __aarch64__
+#define ZIG_NO_F128
+#endif
+
+#ifdef __arm__
+#define ZIG_NO_F128
+#endif
+
+#ifdef __mips__
+#define ZIG_NO_F128
+#endif
+
+#ifdef __riscv
+#define ZIG_NO_F128
+#endif
+
+#ifdef __powerpc__
+#define ZIG_NO_F128
+#endif
+
+#ifdef __APPLE__
+#define ZIG_NO_F128
+#endif
+
 #ifndef ZIG_NO_I128
 struct i128 {
     __int128 value;
@@ -138,6 +195,15 @@ struct SmallStructInts {
 void zig_small_struct_ints(struct SmallStructInts);
 struct SmallStructInts zig_ret_small_struct_ints();
 
+struct MedStructInts {
+    int32_t x;
+    int32_t y;
+    int32_t z;
+};
+
+void zig_med_struct_ints(struct MedStructInts);
+struct MedStructInts zig_ret_med_struct_ints();
+
 struct MedStructMixed {
     uint32_t a;
     float b;
@@ -215,8 +281,10 @@ void run_c_tests(void) {
     }
 #endif
 
+#ifndef ZIG_BUG_14908
     zig_i8(-1);
     zig_i16(-2);
+#endif
     zig_i32(-3);
     zig_i64(-4);
 
@@ -284,14 +352,22 @@ void run_c_tests(void) {
     }
 #endif
 
-#if !defined __i386__ && !defined __arm__ && !defined __mips__ && \
-    !defined ZIG_PPC32 && !defined _ARCH_PPC64
+#if !defined __i386__ && !defined __arm__ && !defined __aarch64__ && \
+    !defined __mips__ && !defined __powerpc__ && !defined ZIG_RISCV64
     {
         struct SmallStructInts s = {1, 2, 3, 4};
         zig_small_struct_ints(s);
     }
 #endif
 
+#if !defined __i386__ && !defined __arm__ && !defined __aarch64__ && \
+    !defined __mips__ && !defined __powerpc__ && !defined ZIG_RISCV64
+    {
+        struct MedStructInts s = {1, 2, 3};
+        zig_med_struct_ints(s);
+    }
+#endif
+
 #ifndef ZIG_NO_I128
     {
         __int128 s = 0;
@@ -531,6 +607,27 @@ struct SmallStructInts c_ret_small_struct_ints() {
     return s;
 }
 
+void c_med_struct_ints(struct MedStructInts s) {
+    assert_or_panic(s.x == 1);
+    assert_or_panic(s.y == 2);
+    assert_or_panic(s.z == 3);
+
+    struct MedStructInts s2 = zig_ret_med_struct_ints();
+
+    assert_or_panic(s2.x == 1);
+    assert_or_panic(s2.y == 2);
+    assert_or_panic(s2.z == 3);
+}
+
+struct MedStructInts c_ret_med_struct_ints() {
+    struct MedStructInts s = {
+        .x = 1,
+        .y = 2,
+        .z = 3,
+    };
+    return s;
+}
+
 void c_med_struct_mixed(struct MedStructMixed x) {
     assert_or_panic(x.a == 1234);
     assert_or_panic(x.b == 100.0f);
@@ -742,6 +839,19 @@ SmallVec c_ret_small_vec(void) {
     return (SmallVec){3, 4};
 }
 
+typedef size_t MediumVec __attribute__((vector_size(4 * sizeof(size_t))));
+
+void c_medium_vec(MediumVec vec) {
+    assert_or_panic(vec[0] == 1);
+    assert_or_panic(vec[1] == 2);
+    assert_or_panic(vec[2] == 3);
+    assert_or_panic(vec[3] == 4);
+}
+
+MediumVec c_ret_medium_vec(void) {
+    return (MediumVec){5, 6, 7, 8};
+}
+
 typedef size_t BigVec __attribute__((vector_size(8 * sizeof(size_t))));
 
 void c_big_vec(BigVec vec) {
@@ -871,3 +981,115 @@ void c_func_ptr_byval(void *a, void *b, struct ByVal in, unsigned long c, void *
     assert_or_panic((intptr_t)d == 4);
     assert_or_panic(e == 5);
 }
+
+#ifndef ZIG_NO_RAW_F16
+__fp16 c_f16(__fp16 a) {
+    assert_or_panic(a == 12);
+    return 34;
+}
+#endif
+
+typedef struct {
+    __fp16 a;
+} f16_struct;
+f16_struct c_f16_struct(f16_struct a) {
+    assert_or_panic(a.a == 12);
+    return (f16_struct){34};
+}
+
+#if defined __x86_64__ || defined __i386__
+typedef long double f80;
+f80 c_f80(f80 a) {
+    assert_or_panic((double)a == 12.34);
+    return 56.78;
+}
+typedef struct {
+    f80 a;
+} f80_struct;
+f80_struct c_f80_struct(f80_struct a) {
+    assert_or_panic((double)a.a == 12.34);
+    return (f80_struct){56.78};
+}
+typedef struct {
+    f80 a;
+    int b;
+} f80_extra_struct;
+f80_extra_struct c_f80_extra_struct(f80_extra_struct a) {
+    assert_or_panic((double)a.a == 12.34);
+    assert_or_panic(a.b == 42);
+    return (f80_extra_struct){56.78, 24};
+}
+#endif
+
+#ifndef ZIG_NO_F128
+__float128 c_f128(__float128 a) {
+    assert_or_panic((double)a == 12.34);
+    return 56.78;
+}
+typedef struct {
+    __float128 a;
+} f128_struct;
+f128_struct c_f128_struct(f128_struct a) {
+    assert_or_panic((double)a.a == 12.34);
+    return (f128_struct){56.78};
+}
+#endif
+
+void __attribute__((stdcall)) stdcall_scalars(char a, short b, int c, float d, double e) {
+    assert_or_panic(a == 1);
+    assert_or_panic(b == 2);
+    assert_or_panic(c == 3);
+    assert_or_panic(d == 4.0);
+    assert_or_panic(e == 5.0);
+}
+
+typedef struct {
+    short x;
+    short y;
+} Coord2;
+
+Coord2 __attribute__((stdcall)) stdcall_coord2(Coord2 a, Coord2 b, Coord2 c) {
+    assert_or_panic(a.x == 0x1111);
+    assert_or_panic(a.y == 0x2222);
+    assert_or_panic(b.x == 0x3333);
+    assert_or_panic(b.y == 0x4444);
+    assert_or_panic(c.x == 0x5555);
+    assert_or_panic(c.y == 0x6666);
+    return (Coord2){123, 456};
+}
+
+void __attribute__((stdcall)) stdcall_big_union(union BigUnion x) {
+    assert_or_panic(x.a.a == 1);
+    assert_or_panic(x.a.b == 2);
+    assert_or_panic(x.a.c == 3);
+    assert_or_panic(x.a.d == 4);
+}
+
+#ifdef __x86_64__
+struct ByRef __attribute__((ms_abi)) c_explict_win64(struct ByRef in) {
+    in.val = 42;
+    return in;
+}
+
+struct ByRef __attribute__((sysv_abi)) c_explict_sys_v(struct ByRef in) {
+    in.val = 42;
+    return in;
+}
+#endif
+
+
+struct byval_tail_callsite_attr_Point {
+    double x;
+    double y;
+} Point;
+struct byval_tail_callsite_attr_Size {
+    double width;
+    double height;
+} Size;
+struct byval_tail_callsite_attr_Rect {
+    struct byval_tail_callsite_attr_Point origin;
+    struct byval_tail_callsite_attr_Size size;
+};
+double c_byval_tail_callsite_attr(struct byval_tail_callsite_attr_Rect in) {
+    return in.size.width;
+}
